const EventEmitter = require('events').EventEmitter
const events = new EventEmitter()
const uuid = require('uuid')

module.exports = sero => {
	function service(topic, fns) {
		Object.keys(fns).forEach(fnkey => {
			let fn = fns[fnkey]
			events.on(`sero-service-${name}.${fnkey}`, async (data) => {
				try {
					let response = await fn(...data.args)
					events.emit(`sero-call-${data.request_id}`, {response, error: false})
				} catch (e) {
					events.emit(`sero-call-${data.request_id}`, {response: e, error: true})
				}
			})
		})
	}
	
	function call(topic, ...data) {
		return new Promise((resolve, reject) => {
			let request_id = uuid.v1()
			events.on(`sero-call-${request_id}`, ({response, error}) => {
				if (error)
					return reject(response)
				resolve(response)
			})
			events.emit(`sero-service-${topic}`, {args: data, request_id})
		})
	}
	
	function emit(topic, ...data) {
		events.emit(`sero-event-${topic}`, {args: data})
	}
	
	function on(topic, callback) {
		events.on(`sero-event-${topic}`, ({args}) => {
			callback(...args)
		})
	}
	
	function once(topic, callback) {
		events.once(`sero-event-${topic}`, ({args}) => {
			callback(...args)
		})
	}
	
	
	
	return {
		service,
		call,
		emit,
		on,
		once
	}
}