const NATS = require('nats')
const uuid = require('uuid')

module.exports = (sero) => {
	const nats = NATS.connect(JSON.parse(process.env.SERO_NATS_CONFIG || '{}'))

	function service(name, fns) {
		Object.keys(fns).forEach(fnkey => {
			let fn = fns[fnkey]
			nats.subscribe(`sero-service-${name}.${fnkey}`, { queue: 'sero-service' }, async (msg) => {
				let data = JSON.parse(msg)
				try {
					let response = await fn(...data.args)
					nats.publish(`sero-call-${data.request_id}`, JSON.stringify({response, error: false}))
				} catch (e) {
					nats.publish(`sero-call-${data.request_id}`, JSON.stringify({response: e, error: true}))
				}
			})
		})
	}
	
	function call(topic, ...data) {
		return new Promise((resolve, reject) => {
			let request_id = uuid.v1()
			let response_sid = nats.subscribe(`sero-call-${request_id}`, (msg) => {
				let {response, error} = JSON.parse(msg)

				nats.unsubscribe(response_sid)

				if (error)
					return reject(response)
				resolve(response)
			})
			nats.publish(`sero-service-${topic}`, JSON.stringify({args: data, request_id}))
		})
	}
	
	function emit(topic, ...data) {
		nats.publish(`sero-event-${topic}`, JSON.stringify({args: data}))
	}
	
	function on(topic, callback) {
		let sid = nats.subscribe(`sero-event-${topic}`, (msg) => {
			let {args} = JSON.parse(msg)
			callback(...args)
		})

		return () => {
			nats.unsubscribe(sid)
		}
	}
	
	function once(topic, callback) {
		let sid = nats.subscribe(`sero-event-${topic}`, (msg) => {
			let {args} = JSON.parse(msg)
			nats.unsubscribe(sid)
			callback(...args)
		})
	}
	
	
	
	return {
		service,
		call,
		emit,
		on,
		once
	}
}